from pathlib import Path
import numpy as np
from chunkhive.retrieval.vector_store import VectorStore

store = VectorStore(Path("data/embeddings/my_index_v2.chunks"))
store.load()

print(store.info())
